## 正则表达式是什么鬼
正则表达式，又称正规表示式、正规表示法、正规表达式、规则表达式、常规表示法（英语：Regular Expression，在代码中常简写为regex、regexp或RE），计算机科学的一个概念。正则表达式使用单个字符串来描述、匹配一系列符合某个句法规则的字符串。在很多文本编辑器里，正则表达式通常被用来检索、替换那些符合某个模式的文本。（以上内容来自维基百科）  

现在是大数据时代，各种处理字符串恐怕是数据工作者最常用的技能了。另外，做网页的，做界面的，做爬虫的同学们，经常会需要查找替换各种复合某些复杂规则的字符串。换句话说，只要跟字符串打交道，就离不开正则表达式的身影。写得一手好正则，能极大地提高工作效率，这也是一个玩数据同志的切身体会。  

So，Let's Go，一起走进正则的奇妙世界吧  

## 正则表达式元字符
<table>
   <tr>
      <td>字符</td>
      <td>描述</td>
   </tr>
   <tr>
      <td>\</td>
      <td>将下一个字符标记为一个特殊字符、或一个原义字符、或一个 后向引用、或一个八进制转义符。例如，'n' 匹配字符 "n"。'\n' 匹配一个换行符。序列 '\\' 匹配 "\" 而 "\(" 则匹配 "("。</td>
   </tr>
   <tr>
      <td>^</td>
      <td>匹配输入字符串的开始位置。如果设置了 RegExp 对象的 Multiline 属性，^ 也匹配 '\n' 或 '\r' 之后的位置。</td>
   </tr>
   <tr>
      <td>$</td>
      <td>匹配输入字符串的结束位置。如果设置了RegExp 对象的 Multiline 属性，$ 也匹配 '\n' 或 '\r' 之前的位置。</td>
   </tr>
   <tr>
      <td>*</td>
      <td>匹配前面的子表达式零次或多次。例如，zo* 能匹配 "z" 以及 "zoo"。 * 等价于{0,}。</td>
   </tr>
   <tr>
      <td>+</td>
      <td>匹配前面的子表达式一次或多次。例如，'zo+' 能匹配 "zo" 以及 "zoo"，但不能匹配 "z"。+ 等价于 {1,}。</td>
   </tr>
   <tr>
      <td>?</td>
      <td>匹配前面的子表达式零次或一次。例如，"do(es)?" 可以匹配 "do" 或 "does" 中的"do" 。? 等价于 {0,1}。</td>
   </tr>
   <tr>
      <td>{n}</td>
      <td>n 是一个非负整数。匹配确定的 n 次。例如，'o{2}' 不能匹配 "Bob" 中的 'o'，但是能匹配 "food" 中的两个 o。</td>
   </tr>
   <tr>
      <td>{n,}</td>
      <td>n 是一个非负整数。至少匹配n 次。例如，'o{2,}' 不能匹配 "Bob" 中的 'o'，但能匹配 "foooood" 中的所有 o。'o{1,}' 等价于 'o+'。'o{0,}' 则等价于 'o*'。</td>
   </tr>
   <tr>
      <td>{n,m}</td>
      <td>m 和 n 均为非负整数，其中m>=n，最少匹配 n 次且最多匹配 m 次。刘， "o{1,3}" 将匹配 "fooooood" 中的前三个 o。'o{0,1}' 等价于 'o?'。请注意在逗号和两个数之间不能有空格。</td>
   </tr>
   <tr>
      <td>?</td>
      <td>当该字符紧跟在任何一个其他限制符 (*, +, ?, {n}, {n,}, {n,m}) 后面时，匹配模式是非贪婪的。非贪婪模式尽可能少的匹配所搜索的字符串，而默认的贪婪模式则尽可能多的匹配所搜索的字符串。例如，对于字符串 "oooo"，'o+?' 将匹配单个 "o"，而 'o+' 将匹配所有 'o'。</td>
   </tr>
   <tr>
      <td>.</td>
      <td>匹配除 "\n" 之外的任何单个字符。要匹配包括 '\n' 在内的任何字符，请使用象 '[.\n]' 的模式。</td>
   </tr>
   <tr>
      <td>(pattern)</td>
      <td>匹配pattern 并获取这一匹配。所获取的匹配可以从产生的 Matches 集合得到，在VBScript 中使用 SubMatches 集合，在Visual Basic Scripting Edition 中则使用 $0…$9 属性。要匹配圆括号字符，请使用 '\(' 或 '\)'。</td>
   </tr>
   <tr>
      <td>(?:pattern)</td>
      <td>匹配 pattern 但不获取匹配结果，也就是说这是一个非获取匹配，不进行存储供以后使用。这在使用 "或" 字符 (|) 来组合一个模式的各个部分是很有用。例如， 'industr(?:y|ies) 就是一个比 'industry|industries' 更简略的表达式。</td>
   </tr>
   <tr>
      <td>(?=pattern)</td>
      <td>正向预查，在任何匹配 pattern 的字符串开始处匹配查找字符串。这是一个非获取匹配，也就是说，该匹配不需要获取供以后使用。例如， 'Windows (?=95|98|NT|2000)' 能匹配 "Windows 2000" 中的 "Windows" ，但不能匹配 "Windows 3.1" 中的 "Windows"。预查不消耗字符，也就是说，在一个匹配发生后，在最后一次匹配之后立即开始下一次匹配的搜索，而不是从包含预查的字符之后开始。</td>
   </tr>
   <tr>
      <td>(?!pattern)</td>
      <td>负向预查，在任何不匹配Negative lookahead matches the search string at any point where a string not matching pattern 的字符串开始处匹配查找字符串。这是一个非获取匹配，也就是说，该匹配不需要获取供以后使用。例如'Windows (?!95|98|NT|2000)' 能匹配 "Windows 3.1" 中的 "Windows"，但不能匹配 "Windows 2000" 中的 "Windows"。预查不消耗字符，也就是说，在一个匹配发生后，在最后一次匹配之后立即开始下一次匹配的搜索，而不是从包含预查的字符之后开始</td>
   </tr>
   <tr>
      <td>x|y</td>
      <td>匹配 x 或 y。例如，'z|food' 能匹配 "z" 或 "food"。'(z|f)ood' 则匹配 "zood" 或 "food"。</td>
   </tr>
   <tr>
      <td>[xyz]</td>
      <td>字符集合。匹配所包含的任意一个字符。例如， '[abc]' 可以匹配 "plain" 中的 'a'。</td>
   </tr>
   <tr>
      <td>[^xyz]</td>
      <td>负值字符集合。匹配未包含的任意字符。例如， '[^abc]' 可以匹配 "plain" 中的'p'。</td>
   </tr>
   <tr>
      <td>[a-z]</td>
      <td>字符范围。匹配指定范围内的任意字符。例如，'[a-z]' 可以匹配 'a' 到 'z' 范围内的任意小写字母字符。</td>
   </tr>
   <tr>
      <td>[^a-z]</td>
      <td>负值字符范围。匹配任何不在指定范围内的任意字符。例如，'[^a-z]' 可以匹配任何不在 'a' 到 'z' 范围内的任意字符。</td>
   </tr>
   <tr>
      <td>\b</td>
      <td>匹配一个单词边界，也就是指单词和空格间的位置。例如， 'er\b' 可以匹配"never" 中的 'er'，但不能匹配 "verb" 中的 'er'。</td>
   </tr>
   <tr>
      <td>\B</td>
      <td>匹配非单词边界。'er\B' 能匹配 "verb" 中的 'er'，但不能匹配 "never" 中的 'er'。</td>
   </tr>
   <tr>
      <td>\cx</td>
      <td>匹配由x指明的控制字符。例如， \cM 匹配一个 Control-M 或回车符。 x 的值必须为 A-Z 或 a-z 之一。否则，将 c 视为一个原义的 'c' 字符。</td>
   </tr>
   <tr>
      <td>\d</td>
      <td>匹配一个数字字符。等价于 [0-9]。</td>
   </tr>
   <tr>
      <td>\D</td>
      <td>匹配一个非数字字符。等价于 [^0-9]。</td>
   </tr>
   <tr>
      <td>\f</td>
      <td>匹配一个换页符。等价于 \x0c 和 \cL。</td>
   </tr>
   <tr>
      <td>\n</td>
      <td>匹配一个换行符。等价于 \x0a 和 \cJ。</td>
   </tr>
   <tr>
      <td>\r</td>
      <td>匹配一个回车符。等价于 \x0d 和 \cM。</td>
   </tr>
   <tr>
      <td>\s</td>
      <td>匹配任何空白字符，包括空格、制表符、换页符等等。等价于 [ \f\n\r\t\v]。</td>
   </tr>
   <tr>
      <td>\S</td>
      <td>匹配任何非空白字符。等价于 [^ \f\n\r\t\v]。</td>
   </tr>
   <tr>
      <td>\t</td>
      <td>匹配一个制表符。等价于 \x09 和 \cI。</td>
   </tr>
   <tr>
      <td>\v</td>
      <td>匹配一个垂直制表符。等价于 \x0b 和 \cK。</td>
   </tr>
   <tr>
      <td>\w</td>
      <td>匹配包括下划线的任何单词字符。等价于'[A-Za-z0-9_]'。</td>
   </tr>
   <tr>
      <td>\W</td>
      <td>匹配任何非单词字符。等价于 '[^A-Za-z0-9_]'。</td>
   </tr>
   <tr>
      <td>\xn</td>
      <td>匹配 n，其中 n 为十六进制转义值。十六进制转义值必须为确定的两个数字长。例如， '\x41' 匹配 "A"。'\x041' 则等价于 '\x04' & "1"。正则表达式中可以使用 ASCII 编码。.</td>
   </tr>
   <tr>
      <td>\num</td>
      <td>匹配 num，其中 num 是一个正整数。对所获取的匹配的引用。例如，'(.)\1' 匹配两个连续的相同字符。</td>
   </tr>
   <tr>
      <td>\n</td>
      <td>标识一个八进制转义值或一个后向引用。如果 \n 之前至少 n 个获取的子表达式，则 n 为后向引用。否则，如果 n 为八进制数字 (0-7)，则 n 为一个八进制转义值。</td>
   </tr>
   <tr>
      <td>\nm</td>
      <td>标识一个八进制转义值或一个后向引用。如果 \nm 之前至少有is preceded by at least nm 个获取得子表达式，则 nm 为后向引用。如果 \nm 之前至少有 n 个获取，则 n 为一个后跟文字 m 的后向引用。如果前面的条件都不满足，若  n 和 m 均为八进制数字 (0-7)，则 \nm 将匹配八进制转义值 nm。</td>
   </tr>
   <tr>
      <td>\nml</td>
      <td>如果 n 为八进制数字 (0-3)，且 m 和 l 均为八进制数字 (0-7)，则匹配八进制转义值 nml。</td>
   </tr>
   <tr>
      <td>\un</td>
      <td>匹配 n，其中 n 是一个用四个十六进制数字表示的 Unicode 字符。例如， \u00A9 匹配版权符号 (?)。</td>
   </tr>
</table>



元字符比较多。不过不用着急，先快速浏览一遍，后续用到的时候再各个击破  

## 正则表达式初探
正则的语法比较多，也比较晦涩难懂。所以个人观点，正则的最好学习方式，就是从实例开始，从需求开始，回头再去理解相应的语法，这样学习起来比较简单  

像coding的时候最常见的场景，想查找比如print这个单词，一般的编辑器，IDE都是ctrl+f，然后在find里输入'print'，然后就能找到所有包含'print'这个字符串的位置。  

这里有几个问题：  
1.我们可能想做的不是精确匹配'print'，也可能想匹配'prinT,'Print','prInt'等诸多情况。这种情况下，一般指定re.I模式，就可以匹配到上述所有字符串。'I'选项，在正则中一般就是'Ignore'的意思，忽略大小写。  
2.我们上述匹配出来的字符串，比如在java代码里，一般包含'println'的情况最多。但是很多时候，我们是想精确匹配某一个字符串或者单词，比如我们并不想匹配'print'。这个时候，元字符就派上用场了。'\bprint\b'可以达到我们的预期效果。  