## 1.卡方分布
卡方分布(chi-square distribution, $\chi ^2$-distribution)是概率统计里常用的一种概率分布，也是统计推断里应用最广泛的概率分布之一，在假设检验与置信区间的计算中经常能见到卡方分布的身影。  

我们先来看看卡方分布的定义：  
若$k$个独立的随机变量$Z_1,Z_2,\cdots,Z_k$，且符合标准正态分布$N(0,1)$，则这$k$个随机变量的平方和  
$$X = \sum_{i=1}^k Z_i^2$$  
为服从自由度为$k$的卡方分布，记为：  
$$X \sim \chi^2(k)$$  
也可以记为：  
$$X \sim \chi_k^2$$  

卡方分布的期望与方差分为为：  
$E(\chi^2) = n$，$D(\chi^2) = 2n$，其中$n$为卡方分布的自由度。

## 2.卡方检验
$\chi^2$检验是以$\chi^2$分布为基础的一种假设检验方法，主要用于分类变量。其基本思想是根据样本数据推断总体的分布与期望分布是否有显著性差异，或者推断两个分类变量是否相关或者独立。  
一般可以设原假设为 $H_0$：观察频数与期望频数没有差异，或者两个变量相互独立不相关。  
实际应用中，我们先假设$H_0$成立，计算出$\chi^2$的值，$\chi^2$表示观察值与理论值之间的偏离程度。根据$\chi^2$分布，$\chi^2$统计量以及自由度，可以确定在$H_0$成立的情况下获得当前统计量以及更极端情况的概率p。如果p很小，说明观察值与理论值的偏离程度大，应该拒绝原假设。否则不能拒绝原假设。  

$\chi^2$的计算公式为：  
$$\chi^2 = \sum \frac{(A-T)^2}{T}$$  
其中，A为实际值，T为理论值。  

$\chi^2$用于衡量实际值与理论值的差异程度，这也是卡方检验的核心思想。$\chi^2$包含了以下两个信息：  
1.实际值与理论值偏差的绝对大小。  
2.差异程度与理论值的相对大小。  

## 3.卡方检验做特征选择
卡方检验经常被用来做特征选择。举个网络上的例子，假设我们有一堆新闻标题，需要判断标题中包含某个词（比如吴亦凡）是否与该条新闻的类别归属（比如娱乐）是否有关，我们只需要简单统计就可以获得这样的一个四格表：  

<table>
<thead><tr>
<th>组别</th>
<th>属于<code>娱乐</code>
</th>
<th>不属于<code>娱乐</code>
</th>
<th>合计</th>
</tr></thead>
<tbody>
<tr>
<td>不包含<code>吴亦凡</code>
</td>
<td>19</td>
<td>24</td>
<td>43</td>
</tr>
<tr>
<td>包含<code>吴亦凡</code>
</td>
<td>34</td>
<td>10</td>
<td>44</td>
</tr>
<tr>
<td>合计</td>
<td>53</td>
<td>34</td>
<td>87</td>
<td>　</td>
</tr>
</tbody>
</table>

通过这个四格表我们得到的第一个信息是：标题是否包含吴亦凡确实对新闻是否属于娱乐有统计上的差别，包含吴亦凡的新闻属于娱乐的比例更高，但我们还无法排除这个差别是否由于抽样误差导致。那么首先假设标题是否包含吴亦凡与新闻是否属于娱乐是独立无关的，随机抽取一条新闻标题，属于娱乐类别的概率是：(19 + 34) / (19 + 34 + 24 +10) = 60.9%  

理论值的四格表为：  
<table>
<thead><tr>
<th>组别</th>
<th>属于<code>娱乐</code>
</th>
<th>不属于<code>娱乐</code>
</th>
<th>合计</th>
</tr></thead>
<tbody>
<tr>
<td>不包含<code>吴亦凡</code>
</td>
<td>43 * 0.609 = 26.2</td>
<td>43 * 0.391 = 16.8</td>
<td>43</td>
</tr>
<tr>
<td>包含<code>吴亦凡</code>
</td>
<td>44 * 0.609 = 26.8</td>
<td>44 * 0.391 = 17.2</td>
<td>44</td>
</tr>
</tbody>
</table>

显然，如果两个变量是独立无关的，那么四格表中的理论值与实际值的差异会非常小。  

则$\chi^2$值为:  
$$\chi^2 = \frac{(19-26.2)^2}{26.2} +  \frac{(34-26.8)^2}{26.8} + \frac{(24-16.8)^2}{16.8} + \frac{(10-17.2)^2}{17.2} = 10.00$$  

标准的四格表$\chi^2$值可以用以下方式进行计算：  
$$\chi^2 = \frac{N * (AD-BC)^2}{(A+B)(C+D)(A+C)(B+D)}$$  
其中，$N = A+B+C+D$  

得到$\chi^2$的值以后，怎样可以得知无关性假设是否可靠？接下来我们应该查询卡方分布的临界值表了。  

首先我们明确自由度的概念：自由度v=(行数-1)*(列数-1)。  
然后看卡方分布的临界概率，表如下：  
![这里写图片描述](https://github.com/bitcarmanlee/easy-algorithm-interview-photo/blob/master/feature-engineering/chi/chi.png)

一般我们取p=0.05，也就是说两者不相关的概率为0.05时，对应的卡方值为3.84。显然10.0>3.84，那就说明包含吴亦凡的新闻不属于娱乐的概率小于0.05。换句话说，包含吴亦凡的新闻与娱乐新闻相关的概率大于95%！  

总结一下：我们可以通过卡方值来判断特征是否与类型有关。卡方值越大，说明关联越强，特征越需要保留。卡方值越小，说明越不相关，特征需要去除。
